/* This file is part of Cloudy and is copyright (C)1978-2014 by Gary J. Ferland and
 * others.  For conditions of distribution and use see copyright notice in license.txt */
/*save_colden parse save column density command, or actually do the save lines output */
#include "cddefines.h"
#include "cddrive.h"
#include "mole.h"
#include "parser.h"

class SpeciesName
{
public:
	char chElement[NCHLAB];
	long int ionstage;
	string chLabel;
};

static vector<SpeciesName> savecoldata;

/*save_colden parse save column density command, or actually do the save lines output */
void parse_save_colden(Parser &p,
		       /* the header for the file, a list of identifications */
		       ostringstream& chHeader)
{
	DEBUG_ENTRY( "parse_save_colden()" );

	/* very first time this routine is called, chDo is "READ" and we read
	 * in lines from the input stream.  The line labels and wavelengths
	 * are store locally, and output in later calls to this routine */
	
	/* number of lines we will save */
	savecoldata.resize(0);
	
	/* get the next line, and check for eof */
	p.getline();
	if( p.m_lgEOF )
	{
		fprintf( ioQQQ, 
					" Hit EOF while reading line list; use END to end list.\n" );
		cdEXIT(EXIT_FAILURE);
	}
	
	while( ! p.hasCommand( "END" ) )
	{
		savecoldata.resize(savecoldata.size()+1);
		/* order on line is label (col 1-4), savecoldata.ionstage */
		SpeciesName& next = savecoldata[savecoldata.size()-1];

		next.chLabel = "";

		if (!p.GetQuote(next.chLabel))
		{
			next.chElement[0] = '\0';
			next.ionstage = -1;
		}
		else
		{
			strncpy( next.chElement, p.getFirstChunk(NCHLAB-1).c_str() , NCHLAB-1 );
			
			/* null terminate the string*/
			next.chElement[NCHLAB-1] = 0;
		
			/* now get savecoldata.ionstage - 1 for atom, 2 for first ion, etc */
			next.ionstage = (long)p.FFmtRead();
			if( p.lgEOL() )
				p.NoNumb("ion stage");
		}

		/* get next line and check for eof */
		p.getline();
		if( p.m_lgEOF )
		{
			fprintf( ioQQQ, " Hit EOF while reading line list; use END to end list.\n" );
			cdEXIT(EXIT_FAILURE);
		}
		
	}
	
	/*fprintf( ioPUN, "%li lines were entered, they were;\n", 
	  savecoldata.size() );*/
	/* give header line */
	
	for(size_t i=0; i < savecoldata.size(); i++ )
	{
		if ( i != 0 )
			chHeader << '\t';
		if ( savecoldata[i].chLabel != "")
			chHeader << savecoldata[i].chLabel;
		else 
			sncatf( chHeader, "%s %3li", savecoldata[i].chElement , savecoldata[i].ionstage );
	}
	chHeader << '\n';
}

/*save_colden parse save column density command, or actually do the save lines output */
void save_colden(
  /* the file we will write to */
  FILE * ioPUN )
{
	DEBUG_ENTRY( "save_colden()" );

	/* save some column densities  */
	double colden;
	/* save some column column densities command */
	for( size_t i=0; i < savecoldata.size(); i++ )
	{
		if( i )
			fprintf(ioPUN,"\t");
		/* get column density, returns 0 if all ok */
		if( savecoldata[i].chLabel != "")
		{
			colden = findspecieslocal_validate(savecoldata[i].chLabel.c_str())->column;
		}
		else if( cdColm(
				 /* four char string, null terminated, giving the element name */
				 savecoldata[i].chElement, 
				 /* IonStage is ionization stage */
				 savecoldata[i].ionstage, 
				 /* will be column density */
				 &colden) )
		{
			fprintf( ioQQQ, 
						"\n PROBLEM save_colden could not find a column density for "
						"the species with label %s %li \n\n",
						savecoldata[i].chElement , savecoldata[i].ionstage );
			colden = 1.;
		}
		fprintf( ioPUN, "%.4f", log10( MAX2(SMALLFLOAT , colden ) ) );
	}
	fprintf( ioPUN, "\n" );
}

